/* 
 * $Id: phrase_huayu.c,v 1.0, 05/26/2011 16:21:21 Exp$ 
 *
 * phrase_lib.txt 格式:
 *  第一行: SCIM_Phrase_Library_TEXT 
 *  第二行: VERSION_0_6 
 *  第三行: 词组总数(行数)
 *  第四行: 初始化总共字符个数(ucs4_t, 为了速度考虑可以设置一个默认大小)
 *  第五行: relation_map_size(默认为0)
 *  第六行: ... 开始词组
 *    词组\t频率\t属性1,属性2,属性3
 *  例子: 
 *    公海海底海床和平利用特别委员会  0
 *    鶴  0
 *
 * 说明:
 *   在phrase_lib.txt文件中需要包含基本的输入单词. 在原phrase_lib.txt中的
 *    line 135712 开始, 总共161206内容
 *   所以生成内容:
 *     ./phrase_huayu -s /usr/share/scim/pinyin/phrase_lib.txt -l 135712 sys.txt
 *   这里sys.txt是华宇导出的拼音文本格式
 *
 * Copyright (c) 2011 Andrew E. Smith <andrew1972.es@gmail.com>
 * All rights reserved.
 *  
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

#define MAXLINE 1024

#define SCIM_PHRASE_LIB_HEADER "SCIM_Phrase_Library_TEXT"
#define SCIM_PHRASE_LIB_VER "VERSION_0_6"

static void help() {
    fprintf(stderr, "usage: ./phrase_huayu -s phrase_lib.txt -l line <file1>[,<file2>,<file3>]\n");
    fprintf(stderr, "   -s the original phrase_lib.txt\n");
    fprintf(stderr, "   -l the start line number in the original phrase_lib.txt\n");
    fprintf(stderr, "ex: ./phrase_huayu -s /usr/share/scim/pinyin/phrase_lib.txt -l 135712 sys.txt\n");
    exit(1);
}
static inline int is_valid(const char *line) {
    return 1;
}

int main(int argc, char **argv) {
    int i;
    char *fname;
    FILE *fout, *fin;
    char line[MAXLINE];
    char cmd[512];
    char *cp1, *cp2, *cp3;
    size_t len;
    unsigned long records = 0;
    int freq;
    char *orig_fname=NULL;
    int linenum=0, linecount=0;
    long counts=0;

    while((i=getopt(argc, argv, "hs:l:"))!=-1) {
        switch(i) {
            case 'h':
                help();
                break;
            case 's':
                orig_fname = optarg;  
                break;
            case 'l':
                linenum = atoi(optarg);
                break;
            default:
                help();
        } 
    }
    if(orig_fname==NULL || linenum<=0
            || optind>=argc) {
        help();
    }
    if(!(fout=fopen("phrase_lib.txt", "w"))) {
        fprintf(stderr, "open phrase_lib.txt error: %s\n",
                strerror(errno));
        exit(1); 
    }
    /* 计算总共需要的行数 */
    for(i=optind; i<argc; ++i) {
        fname = argv[i];
        if(fname[0]=='-') help();

        snprintf(cmd, 512, "iconv -f UCS-2LE -t UTF-8 %s",
                        fname);
        if(!(fin=popen(cmd, "r"))) {
            fprintf(stderr, "popen: %s\n", strerror(errno));
            goto end; 
        }
        while(fgets(line, MAXLINE, fin)) {
            if(!memcmp(line, "iconv", 5)) {
                fprintf(stderr, "%s error\n", cmd);
                exit(1);
            }
            len = strlen(line);
            if(!is_valid(line)) continue;

            if(line[len-1]=='\n') line[--len] = '\0';
            if(line[len-1]=='\r') line[--len] = '\0';

            cp1 = cp2 = cp3 = line;

            if(*cp1=='\0' || *cp1==' ' || *cp1=='\t') continue;

            while(*cp2!=' ' && *cp2!='\t' && *cp2!='\0') cp2++;
            if(*cp2=='\0') continue;
            *cp2++='\0';

            while(*cp2==' ' || *cp2=='\t') cp2++;
            if(*cp2=='\0') continue;

            cp3 = cp2;  
            while(*cp3!=' ' && *cp3!='\t' && *cp3!='\0') cp3++;
            if(*cp3=='\0') continue;


            counts++;
        }
        pclose(fin);
    }
    fprintf(stdout, "huayu words: %ld\n", counts);
    /* 从原读取的内容数 */
    linecount = 0;
    if(!(fin=fopen(orig_fname, "r"))) {
        fprintf(stderr, "open `%s` error: %s\n", 
                orig_fname, strerror(errno));
        help();
    }
    while(fgets(line, MAXLINE, fin)) {
        if(++linecount<linenum) continue;
        if(!memchr(line, '\t', strlen(line))) {
            break;
        }
        counts++;
    } 
    fclose(fin);
    fprintf(stdout, "total words: %ld\n", counts);

    /* 输出phrase_lib头内容 */
    fprintf(fout, "%s\n", SCIM_PHRASE_LIB_HEADER);
    fprintf(fout, "%s\n", SCIM_PHRASE_LIB_VER);
    fprintf(fout, "%ld\n", counts);
    fprintf(fout, "%ld\n", counts*2);
    fprintf(fout, "%d\n", 0);

    for(i=optind; i<argc; ++i) {
        fname = argv[i];
        if(fname[0]=='-') help();

        snprintf(cmd, 512, "iconv -f UCS-2LE -t UTF-8 %s",
                        fname);
        if(!(fin=popen(cmd, "r"))) {
            fprintf(stderr, "popen: %s\n", strerror(errno));
            goto end; 
        }
        while(fgets(line, MAXLINE, fin)) {
            if(!memcmp(line, "iconv", 5)) {
                fprintf(stderr, "%s error\n", cmd);
                exit(1);
            }
            len = strlen(line);
            if(!is_valid(line)) continue;

            if(line[len-1]=='\n') line[--len] = '\0';
            if(line[len-1]=='\r') line[--len] = '\0';

            cp1 = cp2 = cp3 = line;

            if(*cp1=='\0' || *cp1==' ' || *cp1=='\t') continue;

            while(*cp2!=' ' && *cp2!='\t' && *cp2!='\0') cp2++;
            if(*cp2=='\0') continue;
            *cp2++='\0';

            while(*cp2==' ' || *cp2=='\t') cp2++;
            if(*cp2=='\0') continue;

            cp3 = cp2;  
            while(*cp3!=' ' && *cp3!='\t' && *cp3!='\0') cp3++;
            if(*cp3=='\0') continue;

            
            /* cp1, cp2, cp3*/
            freq = atoi(cp3);

            fprintf(fout, "%s\t%d\t\n", cp1, freq);

            records++;
        }
        pclose(fin);
    }
    /* 从原读取的内容数 */
    linecount = 0;
    if(!(fin=fopen(orig_fname, "r"))) {
        fprintf(stderr, "open `%s` error: %s\n", 
                orig_fname, strerror(errno));
        help();
    }
    while(fgets(line, MAXLINE, fin)) {
        if(++linecount<linenum) continue;
        if(!memchr(line, '\t', strlen(line))) {
            break;
        }
        fprintf(fout, "%s", line);
        records++;
    } 
    fclose(fin);
    fprintf(fout, "\n");

end:
    fclose(fout);
    fprintf(stdout, "total records: %ld\n", records);
    return 0;
}

